"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.evaluate = exports.termOf = exports.isOccurence = exports.getOccurence = exports.makeSwitch = exports.makeFail = exports.makeLeaf = void 0;
const Utils_1 = require("../Utils");
const DecisionTreeCompiler_1 = require("./DecisionTreeCompiler");
const makeLeaf = (action) => ({ type: 'leaf', action });
exports.makeLeaf = makeLeaf;
const makeFail = () => ({ type: 'fail' });
exports.makeFail = makeFail;
const makeSwitch = (occurence, tests) => {
    return {
        type: 'switch',
        occurence,
        tests
    };
};
exports.makeSwitch = makeSwitch;
const getOccurence = (terms, occurence) => {
    let t = terms[occurence.index];
    for (const idx of occurence.pos) {
        t = t.args[idx];
    }
    return t;
};
exports.getOccurence = getOccurence;
const isOccurence = (t) => {
    return t['pos'] !== undefined && t['index'] !== undefined;
};
exports.isOccurence = isOccurence;
const termOf = (terms, occ) => {
    if (exports.isOccurence(occ))
        return exports.getOccurence(terms, occ);
    return {
        name: occ.name,
        args: occ.args.map(arg => exports.termOf(terms, arg))
    };
};
exports.termOf = termOf;
const evaluate = (args, dt) => {
    if (dt.type === 'leaf')
        return exports.termOf(args, dt.action);
    if (dt.type === 'switch') {
        for (const [ctor, subtree] of dt.tests) {
            const term = exports.getOccurence(args, dt.occurence);
            if (ctor === DecisionTreeCompiler_1._ || Utils_1.isFun(term, ctor)) {
                return exports.evaluate(args, subtree);
            }
        }
    }
};
exports.evaluate = evaluate;
