"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecisionTreeTranslator = void 0;
const Lazify_1 = require("../../Compiler/Passes/Lazify");
const Utils_1 = require("../../Parser/Utils");
const Translator_1 = require("../../Translator/Translator");
const Utils_2 = require("../Utils");
const DecisionTreeCompiler_1 = require("./DecisionTreeCompiler");
class DecisionTreeTranslator extends Translator_1.Translator {
    constructor(trs, externals) {
        super(trs, externals);
        this.arities = Lazify_1.collectTRSArities(trs);
        this.signature = new Set(this.arities.keys());
    }
    collectVarNames(t, sigma, newVars, offset = 0, depth = 0, parent) {
        const name = parent ? this.accessSubterm(parent, depth) : newVars[offset];
        if (Utils_2.isVar(t)) {
            sigma[t] = name;
            return;
        }
        t.args.forEach((s, idx) => {
            this.collectVarNames(s, sigma, newVars, offset + idx, idx, name);
        });
    }
    // Rules are compiled to decitions trees and then translated to source code
    translateRules(name, rules) {
        const newVars = Utils_2.genVars(Utils_2.arity(rules));
        const rules_ = rules.map(([lhs, rhs]) => {
            const sigma = {};
            for (const [t, i] of Utils_1.indexed(lhs.args)) {
                this.collectVarNames(t, sigma, newVars, i);
            }
            return [
                Utils_2.substitute(lhs, sigma),
                Utils_2.substitute(rhs, sigma)
            ];
        });
        const m = DecisionTreeCompiler_1.clauseMatrixOf(rules_);
        const dt = DecisionTreeCompiler_1.compileClauseMatrix(Utils_2.arity(rules), m, this.signature);
        return this.translateDecisionTree(name, dt, newVars);
    }
}
exports.DecisionTreeTranslator = DecisionTreeTranslator;
