"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeSimSuffixes = exports.replaceVars = exports.leftLinearize = exports.symbolNameSimulationSuffix = void 0;
const Closure_1 = require("../../Normalizer/Matchers/ClosureMatcher/Closure");
const RuleBasedUnify_1 = require("../../Normalizer/RuleBasedUnify");
const Utils_1 = require("../../Parser/Utils");
const Types_1 = require("../../Types");
const Imports_1 = require("../Passes/Imports");
const Utils_2 = require("../Utils");
const Checks_1 = require("./Checks");
exports.symbolNameSimulationSuffix = /_(sim)[0-9]+/g;
const rulesName = (rules) => Utils_2.ruleName(rules[0]);
// transforms a general TRS into a left-linear one
// requires a structural equality external function (eqSymb)
// should be used with a normalizeLhsArgs pass
const leftLinearize = (trs) => {
    const symbs = Closure_1.collectSymbols(Utils_2.funs(trs));
    const notLeftLinear = Utils_1.find(trs.keys(), name => trs.get(name).some(rule => !Checks_1.isLeftLinear(rule)));
    if (notLeftLinear) {
        const rules = trs.get(notLeftLinear);
        for (const [rule, index] of Utils_1.indexed(rules)) {
            if (!Checks_1.isLeftLinear(rule)) {
                const [prev, rem] = Utils_2.split(rules, index);
                const { updatedRule, simRules, selectRules, unchangedRules } = leftLinearizeRule(rule, Utils_2.tail(rem), symbs);
                trs.set(notLeftLinear, [...prev, updatedRule, ...unchangedRules]);
                if (selectRules.length > 0) {
                    trs.set(rulesName(selectRules), selectRules);
                }
                if (simRules.length > 0) {
                    trs.set(rulesName(simRules), simRules);
                }
            }
        }
        Imports_1.useAnd(trs);
        // left linearize the updated TRS
        return exports.leftLinearize(trs);
    }
    return Types_1.Ok(trs);
};
exports.leftLinearize = leftLinearize;
const leftLinearizeRule = ([lhs, rhs], remainingRules, // less specific rules with the same head symbol
symbs) => {
    const lhsVars = Utils_2.vars(lhs);
    const rhsVars = Utils_2.vars(rhs);
    const uniqueVars = Utils_2.genVars(lhsVars.length);
    const counts = Utils_2.occurrences(lhsVars);
    const eqs = [];
    for (const occs of counts.values()) {
        if (occs.length > 1) {
            eqs.push(allEq(occs.map(idx => uniqueVars[idx])));
        }
    }
    const newRhsVars = rhsVars.map(v => uniqueVars[lhsVars.indexOf(v)]);
    const simName = Utils_2.freshPrefixedSymb(`${lhs.name}_sim`, symbs);
    const newLhs = replaceVars(lhs, uniqueVars);
    const selectRuleName = Utils_2.freshPrefixedSymb(`${lhs.name}_select`, symbs);
    const updatedRule = [newLhs, Utils_2.fun(selectRuleName, conjunction(eqs), ...newLhs.args)];
    const selectRules = [
        [Utils_2.fun(selectRuleName, Imports_1.True, ...newLhs.args), replaceVars(rhs, newRhsVars)],
        [Utils_2.fun(selectRuleName, Imports_1.False, ...newLhs.args), Utils_2.fun(simName, ...newLhs.args)],
    ];
    // only change the head symbol of matching rules
    const [matchingRules, unchangedRules] = Utils_1.partition(remainingRules, ([lhs2, _]) => Utils_2.isSomething(RuleBasedUnify_1.ruleBasedUnify(lhs, lhs2)));
    return {
        updatedRule,
        simRules: matchingRules.map(([lhs, rhs]) => Utils_2.rule(Utils_2.fun(simName, ...lhs.args), rhs)),
        selectRules,
        unchangedRules
    };
};
function replaceVars(t, newVars, i = { offset: 0 }) {
    if (Utils_2.isVar(t))
        return newVars[i.offset++];
    return Utils_2.fun(t.name, ...t.args.map(s => replaceVars(s, newVars, i)));
}
exports.replaceVars = replaceVars;
const conjunction = (terms) => {
    if (terms.length === 0)
        return Imports_1.True;
    if (terms.length === 1)
        return Utils_2.head(terms);
    const [h, tl] = Utils_2.decons(terms);
    return Imports_1.And(h, conjunction(tl));
};
// allEq(['a', 'b', 'c']) = And(Eq('a', 'b'), Eq('b', 'c'))
const allEq = (terms) => {
    if (terms.length <= 1)
        return Imports_1.True;
    if (terms.length === 2)
        return Imports_1.Eq(terms[0], terms[1]);
    const [h, tl] = Utils_2.decons(terms);
    return Imports_1.And(Imports_1.Eq(h, Utils_2.head(tl)), allEq(tl));
};
const removeSimSuffixes = (term) => {
    return term.replace(exports.symbolNameSimulationSuffix, '');
};
exports.removeSimSuffixes = removeSimSuffixes;
