"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeRhs = void 0;
const Normalizer_1 = require("../../Normalizer/Normalizer");
const RuleBasedUnify_1 = require("../../Normalizer/RuleBasedUnify");
const UndirectedUnify_1 = require("../../Normalizer/UndirectedUnify");
const Types_1 = require("../../Types");
const Utils_1 = require("../Utils");
// requires 'uniqueVarNames' to be used before running this pass
const normalizeRhs = (steps, allowDuplication = false, exceptions = new Set()) => {
    return (trs) => {
        const newRules = [];
        const stepNorm = {
            oneStepReduce: (q) => {
                if (Utils_1.isVar(q))
                    return q;
                if (q.name.startsWith('@') || exceptions.has(q.name)) {
                    return q;
                }
                if (!trs.has(q.name))
                    return;
                const matchingRules = trs.get(q.name)
                    .filter(([lhs, _]) => Utils_1.isSomething(UndirectedUnify_1.undirectedUnify(lhs, q)));
                // only normalize if there's exactly one rule that can possibly match
                if (matchingRules.length === 1) {
                    const [lhs, rhs] = matchingRules[0];
                    if (allowDuplication || !Utils_1.hasDuplicates(Utils_1.vars(rhs))) {
                        const sigma = RuleBasedUnify_1.ruleBasedUnify(lhs, q);
                        if (sigma) {
                            const s = Utils_1.substitute(rhs, sigma);
                            return s;
                        }
                    }
                }
                return q;
            }
        };
        const norm = Normalizer_1.buildFueledNormalizer(stepNorm, steps, {}, false);
        for (const [lhs, rhs] of Utils_1.rules(trs)) {
            newRules.push([lhs, norm(rhs)]);
        }
        return Types_1.Ok(Utils_1.mapify(newRules));
    };
};
exports.normalizeRhs = normalizeRhs;
