"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.swap = exports.swapMut = exports.setEq = exports.setDiff = exports.hasRecursiveRule = exports.isRuleTailRecursive = exports.isRuleRecursive = exports.hasMostGeneralRule = exports.isRuleMostGeneral = exports.mostGeneralFun = exports.genVars = exports.fun = exports.ruleArity = exports.arity = exports.emptyTRS = exports.cloneTRS = exports.showSubst = exports.showRule = exports.showTerm = exports.showTermRec = exports.showTRS = exports.unmapify = exports.mapify = exports.removeRules = exports.ruleVars = exports.uniq = exports.rhs = exports.lhs = exports.ruleName = exports.addRules = exports.hasRule = exports.rulesEq = exports.zip = exports.termsEq = exports.substitute = exports.isSomething = exports.isNothing = exports.occurs = exports.allSymbs = exports.funs = exports.terms = exports.rule = exports.rules = exports.vars = exports.isConst = exports.isVar = exports.isFun = exports.defaultPasses = exports.logTRS = exports.defined = void 0;
exports.associate = exports.stringifyQueryVars = exports.replaceSubstrings = exports.freshPrefixedSymb = exports.rulesAlphaEquiv = exports.alphaEquiv = exports.isEmpty = exports.replaceTermAt = exports.cloneTerm = exports.replaceTerms = exports.trd = exports.snd = exports.fst = exports.split = exports.decons = exports.tail = exports.last = exports.head = exports.hasDuplicatesMap = exports.hasDuplicatesSet = exports.hasDuplicates = exports.elem = exports.occurrences = exports.fill = exports.unusedRuleVars = void 0;
const Types_1 = require("../Parser/Types");
const Utils_1 = require("../Parser/Utils");
const Types_2 = require("../Types");
const Checks_1 = require("./Passes/Checks");
const Lazify_1 = require("./Passes/Lazify");
const LeftLinearize_1 = require("./Passes/LeftLinearize");
const OrderBy_1 = require("./Passes/OrderBy");
const NormalizeLhsArgs_1 = require("./Passes/NormalizeLhsArgs");
const SimulateIfs_1 = require("./Passes/SimulateIfs");
// Used when we know for sure that a maybe is defined
function defined(mb) {
    if (isNothing(mb)) {
        throw new Error(`Called 'defined' on Nothing`);
    }
    return mb;
}
exports.defined = defined;
const logTRS = (trs) => {
    console.log(exports.showTRS(trs));
    return Types_2.Ok(trs);
};
exports.logTRS = logTRS;
const defaultPasses = exts => [
    Checks_1.check(Checks_1.checkNoFreeVars, Checks_1.checkArity, Checks_1.checkNoDuplicates),
    Checks_1.warn((warnings) => warnings.forEach(w => console.warn(w))),
    // currify,
    OrderBy_1.orderBySpecificity,
    LeftLinearize_1.leftLinearize,
    SimulateIfs_1.simulateIfs(),
    Lazify_1.lazify,
    NormalizeLhsArgs_1.normalizeLhsArgs(exts),
    // logTRS,
];
exports.defaultPasses = defaultPasses;
function isFun(term, name) {
    if (name) {
        return typeof term === "object" && term.name === name;
    }
    return typeof term === "object";
}
exports.isFun = isFun;
function isVar(term, name) {
    if (name) {
        return term === name;
    }
    return typeof term === "string";
}
exports.isVar = isVar;
const isConst = (term) => {
    return isFun(term) && term.args.length === 0;
};
exports.isConst = isConst;
function vars(term, acc = []) {
    if (isVar(term)) {
        acc.push(term);
        return acc;
    }
    return term.args.reduce((acc, t) => vars(t, acc), acc);
}
exports.vars = vars;
function* rules(trs) {
    for (const rules of trs.values()) {
        yield* rules;
    }
}
exports.rules = rules;
const rule = (lhs, rhs) => [lhs, rhs];
exports.rule = rule;
const terms = (trs) => {
    const terms = [];
    for (const rules of trs.values()) {
        for (const [lhs, rhs] of rules) {
            terms.push(lhs, rhs);
        }
    }
    return terms;
};
exports.terms = terms;
const funs = (trs) => {
    const terms = [];
    for (const rules of trs.values()) {
        for (const [lhs, rhs] of rules) {
            terms.push(lhs);
            if (isFun(rhs)) {
                terms.push(rhs);
            }
        }
    }
    return terms;
};
exports.funs = funs;
// returns all the funs in t
const allSymbs = (t, acc = []) => {
    if (isVar(t))
        return acc;
    acc.push(t.name);
    for (const arg of t.args) {
        exports.allSymbs(arg, acc);
    }
    return acc;
};
exports.allSymbs = allSymbs;
function occurs(x, t) {
    return vars(t).includes(x);
}
exports.occurs = occurs;
function isNothing(m) {
    return typeof m === "undefined";
}
exports.isNothing = isNothing;
function isSomething(m) {
    return !isNothing(m);
}
exports.isSomething = isSomething;
function substitute(t, sigma) {
    var _a;
    if (isVar(t))
        return (_a = Types_1.dictGet(sigma, t)) !== null && _a !== void 0 ? _a : t;
    return { name: t.name, args: t.args.map(s => substitute(s, sigma)) };
}
exports.substitute = substitute;
;
const termsEq = (a, b) => {
    if (isVar(a) && isVar(b))
        return a === b;
    if (isFun(a) && isFun(b)) {
        return a.name === b.name &&
            a.args.length === b.args.length &&
            Utils_1.every(zip(a.args, b.args), ([s, t]) => exports.termsEq(s, t));
    }
    return false;
};
exports.termsEq = termsEq;
function* zip(as, bs) {
    const len = Math.min(as.length, bs.length);
    for (let i = 0; i < len; i++) {
        yield [as[i], bs[i]];
    }
}
exports.zip = zip;
const rulesEq = ([lhs1, rhs1], [lhs2, rhs2]) => {
    return exports.termsEq(lhs1, lhs2) && exports.termsEq(rhs1, rhs2);
};
exports.rulesEq = rulesEq;
const hasRule = (trs, rule) => {
    const name = exports.ruleName(rule);
    if (trs.has(name)) {
        return trs.get(name).some(def => exports.rulesEq(def, rule));
    }
    return false;
};
exports.hasRule = hasRule;
// adds a rule to a trs if it does not already exist
const addRules = (trs, ...rules) => {
    for (const rule of rules) {
        const name = exports.ruleName(rule);
        if (!exports.hasRule(trs, rule)) {
            if (trs.has(name)) {
                trs.get(name).push(rule);
            }
            else {
                trs.set(name, [rule]);
            }
        }
    }
};
exports.addRules = addRules;
const ruleName = ([lhs, _]) => lhs.name;
exports.ruleName = ruleName;
const lhs = ([lhs, _]) => lhs;
exports.lhs = lhs;
const rhs = ([_, rhs]) => rhs;
exports.rhs = rhs;
function uniq(vals) {
    return [...new Set(vals)];
}
exports.uniq = uniq;
const ruleVars = ([lhs, rhs]) => {
    return uniq([...vars(lhs), ...vars(rhs)]);
};
exports.ruleVars = ruleVars;
const removeRules = (trs, ...rules) => {
    for (const rule of rules) {
        const name = exports.ruleName(rule);
        if (trs.has(name)) {
            trs.set(name, trs.get(name).filter(def => !exports.rulesEq(rule, def)));
        }
    }
};
exports.removeRules = removeRules;
const mapify = (rules, trs = new Map()) => {
    for (const rule of rules) {
        const f = exports.ruleName(rule);
        if (trs.has(f)) {
            trs.get(f).push(rule);
        }
        else {
            trs.set(f, [rule]);
        }
    }
    return trs;
};
exports.mapify = mapify;
const unmapify = (trs) => ([].concat(...trs.values()));
exports.unmapify = unmapify;
const showTRS = (trs) => {
    const out = [];
    for (const [, rules] of trs) {
        for (const rule of rules) {
            out.push(exports.showRule(rule));
        }
    }
    return out.join("\n");
};
exports.showTRS = showTRS;
const showTermRec = (t) => {
    if (isVar(t))
        return t;
    if (t.args.length === 0)
        return t.name;
    return `${t.name}(${t.args.map(exports.showTermRec).join(", ")})`;
};
exports.showTermRec = showTermRec;
// Requires constructor arities to be consistant
function showTerm(term) {
    if (isVar(term))
        return term;
    if (term.args.length === 0)
        return term.name;
    const terms = [term];
    const stack = [];
    const symbols = [];
    // flatten the terms onto a stack
    while (terms.length > 0) {
        const t = terms.pop();
        if (isVar(t)) {
            stack.push(t);
        }
        else if (t.args.length === 0) {
            stack.push(t.name);
        }
        else {
            for (let i = t.args.length - 1; i >= 0; i--) {
                terms.push(t.args[i]);
            }
            terms.push(t.name);
            symbols.push([t.name, t.args.length]);
        }
    }
    const argsStack = [];
    // assemble constructors back when all arguments have been stringified
    for (let i = stack.length - 1; i >= 0; i--) {
        const t = stack[i];
        if (symbols.length === 0)
            break;
        const [f, ar] = symbols[symbols.length - 1];
        if (t === f) {
            const args = [];
            for (let k = 0; k < ar; k++) {
                args.push(argsStack.pop());
            }
            argsStack.push(f + '(' + args.join(', ') + ')');
            symbols.pop();
        }
        else {
            argsStack.push(t);
        }
    }
    return argsStack[0];
}
exports.showTerm = showTerm;
const showRule = ([lhs, rhs]) => (`${showTerm(lhs)} -> ${showTerm(rhs)}`);
exports.showRule = showRule;
const showSubst = (sigma) => (Types_1.dictEntries(sigma)
    .map(([a, b]) => `${showTerm(a)}: ${showTerm(b)}`)
    .join(", "));
exports.showSubst = showSubst;
const cloneTRS = (trs) => {
    return new Map(trs.entries());
};
exports.cloneTRS = cloneTRS;
const emptyTRS = () => {
    return new Map();
};
exports.emptyTRS = emptyTRS;
const arity = (rules) => {
    return exports.ruleArity(rules[0]);
};
exports.arity = arity;
const ruleArity = ([lhs, _]) => {
    return lhs.args.length;
};
exports.ruleArity = ruleArity;
const fun = (name, ...args) => {
    return { name, args };
};
exports.fun = fun;
function genVars(n) {
    return [...Utils_1.gen(n, i => `v${i + 1}`)];
}
exports.genVars = genVars;
const mostGeneralFun = (name, arity) => {
    return { name, args: genVars(arity) };
};
exports.mostGeneralFun = mostGeneralFun;
const isRuleMostGeneral = ([lhs, _]) => {
    return lhs.args.every(t => isVar(t));
};
exports.isRuleMostGeneral = isRuleMostGeneral;
const hasMostGeneralRule = (rules) => {
    return rules.some(exports.isRuleMostGeneral);
};
exports.hasMostGeneralRule = hasMostGeneralRule;
const isRuleRecursive = ([lhs, rhs]) => {
    const name = exports.ruleName([lhs, rhs]);
    return Utils_1.some(Utils_1.traverseNames(rhs), f => f === name);
};
exports.isRuleRecursive = isRuleRecursive;
const isRuleTailRecursive = ([lhs, rhs]) => {
    const name = exports.ruleName([lhs, rhs]);
    return isFun(rhs, name);
};
exports.isRuleTailRecursive = isRuleTailRecursive;
const hasRecursiveRule = (rules) => (rules.some(exports.isRuleRecursive));
exports.hasRecursiveRule = hasRecursiveRule;
function setDiff(as, bs) {
    const diff = new Set();
    for (const a of as) {
        if (!bs.has(a)) {
            diff.add(a);
        }
    }
    return diff;
}
exports.setDiff = setDiff;
function setEq(as, bs) {
    return as.size === bs.size && !Utils_1.some(as.keys(), a => !bs.has(a));
}
exports.setEq = setEq;
function swapMut(vals, i, j) {
    if (i < 0 || j < 0 || i >= vals.length || j >= vals.length) {
        throw new Error(`invalid swap indices, len: ${vals.length}, i: ${i}, j: ${j}`);
    }
    const tmp = vals[i];
    vals[i] = vals[j];
    vals[j] = tmp;
}
exports.swapMut = swapMut;
function swap(vals, i, j) {
    const copy = [...vals];
    swapMut(copy, i, j);
    return copy;
}
exports.swap = swap;
const unusedRuleVars = ([lhs, rhs]) => {
    return setDiff(new Set(vars(lhs)), new Set(vars(rhs)));
};
exports.unusedRuleVars = unusedRuleVars;
function fill(val, count) {
    const vals = [];
    for (let i = 0; i < count; i++) {
        vals.push(val);
    }
    return vals;
}
exports.fill = fill;
function occurrences(vals) {
    const occurrences = new Map();
    vals.forEach((val, idx) => {
        if (occurrences.has(val)) {
            occurrences.get(val).push(idx);
        }
        else {
            occurrences.set(val, [idx]);
        }
    });
    return occurrences;
}
exports.occurrences = occurrences;
function elem(value, elems, eq = (a, b) => a === b) {
    return elems.some(val => eq(val, value));
}
exports.elem = elem;
function hasDuplicates(vals, eq) {
    if (isEmpty(vals))
        return false;
    const [h, tl] = decons(vals);
    return elem(h, tl, eq) || hasDuplicates(tl);
}
exports.hasDuplicates = hasDuplicates;
function hasDuplicatesSet(vals) {
    return new Set(vals).size !== vals.length;
}
exports.hasDuplicatesSet = hasDuplicatesSet;
function hasDuplicatesMap(vals) {
    return Utils_1.some(occurrences(vals).values(), (occs) => occs.length > 1);
}
exports.hasDuplicatesMap = hasDuplicatesMap;
function head(list) {
    return list[0];
}
exports.head = head;
function last(list) {
    return list[list.length - 1];
}
exports.last = last;
function tail(list) {
    return list.slice(1);
}
exports.tail = tail;
function decons(list) {
    return [head(list), tail(list)];
}
exports.decons = decons;
function split(list, splitIdx) {
    return [list.slice(0, splitIdx), list.slice(splitIdx)];
}
exports.split = split;
function fst([u,]) {
    return u;
}
exports.fst = fst;
function snd([, v]) {
    return v;
}
exports.snd = snd;
function trd([, , w]) {
    return w;
}
exports.trd = trd;
const replaceTerms = (old, by, inside) => {
    return inside.map(t => exports.termsEq(t, old) ? by : (isVar(t) ? t : exports.fun(t.name, ...exports.replaceTerms(old, by, t.args))));
};
exports.replaceTerms = replaceTerms;
const cloneTerm = (t) => {
    if (isVar(t))
        return t;
    return { name: t.name, args: t.args.map(arg => exports.cloneTerm(arg)) };
};
exports.cloneTerm = cloneTerm;
const replaceTermAt = (parent, t, pos) => {
    if (isVar(parent) || parent.args.length === 0 || pos.length === 0) {
        return exports.cloneTerm(parent);
    }
    const copy = exports.cloneTerm(parent);
    let q = copy;
    for (let i = 0; i < pos.length - 1; i++) {
        q = q.args[pos[i]];
        if (isVar(q))
            return copy;
    }
    q.args[last(pos)] = t;
    return copy;
};
exports.replaceTermAt = replaceTermAt;
function isEmpty(collection) {
    return (Array.isArray(collection) ? collection.length : collection.size) ===
        0;
}
exports.isEmpty = isEmpty;
const alphaEquiv = (s, t) => (isSomething(alphaEquivAux([hasDuplicatesSet(vars(t)) ? [t, s] : [s, t]])));
exports.alphaEquiv = alphaEquiv;
const alphaEquivAux = (eqs, sigma = {}) => {
    if (isEmpty(eqs))
        return sigma;
    const [a, b] = eqs.pop();
    if (isVar(a) && isVar(b)) {
        if (Types_1.dictHas(sigma, a)) {
            if (Types_1.dictGet(sigma, a) === b) {
                return alphaEquivAux(eqs, sigma);
            }
        }
        else {
            return alphaEquivAux(eqs, Types_1.dictSet(sigma, a, b));
        }
    }
    else if (isFun(a) && isFun(b, a.name)) {
        if (a.args.length === b.args.length) {
            eqs.push(...zip(a.args, b.args));
            return alphaEquivAux(eqs, sigma);
        }
    }
};
const rulesAlphaEquiv = (rule1, rule2) => {
    return exports.alphaEquiv(exports.lhs(rule1), exports.lhs(rule2));
};
exports.rulesAlphaEquiv = rulesAlphaEquiv;
const freshPrefixedSymb = (prefix, symbs) => {
    for (const i of Utils_1.range(0, Infinity)) {
        if (!symbs.has(`${prefix}${i}`)) {
            return `${prefix}${i}`;
        }
    }
};
exports.freshPrefixedSymb = freshPrefixedSymb;
const replaceSubstrings = (str, substrings) => {
    let replaced = str;
    for (const [symb, replacement] of Object.entries(substrings)) {
        replaced = replaced.replace(new RegExp(replacement, 'g'), symb);
    }
    return replaced;
};
exports.replaceSubstrings = replaceSubstrings;
// query variables are free
const stringifyQueryVars = (q) => {
    if (isFun(q)) {
        return LeftLinearize_1.replaceVars(q, vars(q).map(v => `"${v}"`));
    }
    return q;
};
exports.stringifyQueryVars = stringifyQueryVars;
// maps every key of a given array to a value to build a dictionary
const associate = (keys, vals) => {
    const dict = {};
    for (const key of keys) {
        dict[key] = vals(key);
    }
    return dict;
};
exports.associate = associate;
