"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecisionTreeNormalizer = void 0;
const DecisionTree_1 = require("../Compiler/DecisionTrees/DecisionTree");
const DecisionTreeCompiler_1 = require("../Compiler/DecisionTrees/DecisionTreeCompiler");
const Lazify_1 = require("../Compiler/Passes/Lazify");
const Utils_1 = require("../Compiler/Utils");
const Normalizer_1 = require("./Normalizer");
class DecisionTreeNormalizer {
    constructor(trs) {
        this.rules = new Map();
        const sig = new Set(Lazify_1.collectTRSArities(trs).keys());
        // Compile each rule into its own decision tree
        for (const [name, rules] of trs) {
            const m = DecisionTreeCompiler_1.clauseMatrixOf(rules);
            const dt = DecisionTreeCompiler_1.compileClauseMatrix(Utils_1.arity(rules), m, sig);
            this.rules.set(name, dt);
        }
    }
    oneStepReduce(term) {
        if (Utils_1.isVar(term))
            return term;
        const dt = this.rules.get(term.name);
        if (dt) {
            return DecisionTree_1.evaluate(term.args, dt);
        }
    }
    asNormalizer(externals) {
        return Normalizer_1.buildNormalizer(this, externals);
    }
    asFueledNormalizer(maxSteps, externals) {
        return Normalizer_1.buildFueledNormalizer(this, maxSteps, externals);
    }
}
exports.DecisionTreeNormalizer = DecisionTreeNormalizer;
