"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeQuery = exports.makeNormalizerAsync = exports.buildFueledNormalizer = exports.buildNormalizer = exports.traceNormalize = exports.normalizeWithFuel = exports.normalize = exports.oneStepReduceWithExternals = void 0;
const Utils_1 = require("../Compiler/Utils");
const Types_1 = require("../Parser/Types");
const Utils_2 = require("../Parser/Utils");
const Unification_1 = require("./Unification");
// handles externals
const oneStepReduceWithExternals = (normalizer, externals = {}) => {
    return {
        oneStepReduce: (query) => {
            if (query.name[0] === '@') {
                const f = query.name.substr(1);
                if (Types_1.dictHas(externals, f)) {
                    const newTerm = externals[f](query, normalizer, externals);
                    // prevent infinite loops
                    if (newTerm !== query) {
                        return newTerm;
                    }
                }
                else {
                    throw new Error(`Unknown external function: "${f}"`);
                }
            }
            return normalizer.oneStepReduce(query);
        }
    };
};
exports.oneStepReduceWithExternals = oneStepReduceWithExternals;
const normalize = (query, normalizer, externals, useExternals = true) => {
    let reduced = query;
    const reducer = useExternals ?
        exports.oneStepReduceWithExternals(normalizer, externals) :
        normalizer;
    while (true) {
        if (Utils_1.isVar(reduced))
            return reduced;
        Utils_2.mapMut(reduced.args, s => exports.normalize(s, normalizer, externals, useExternals));
        const newTerm = reducer.oneStepReduce(reduced);
        if (newTerm === undefined)
            return reduced;
        reduced = newTerm;
    }
};
exports.normalize = normalize;
// Reduces a term up to maxSteps times
const normalizeWithFuel = (query, normalizer, externals, maxSteps, useExternals = true) => {
    return normalizeWithFuelAux(query, normalizer, externals, { steps: maxSteps - 1 }, useExternals);
};
exports.normalizeWithFuel = normalizeWithFuel;
const normalizeWithFuelAux = (query, normalizer, externals, maxSteps, useExternals = true) => {
    let reduced = query;
    const reducer = useExternals ?
        exports.oneStepReduceWithExternals(normalizer, externals) :
        normalizer;
    while (maxSteps.steps >= 0) {
        if (Utils_1.isVar(reduced))
            return reduced;
        Utils_2.mapMut(reduced.args, s => normalizeWithFuelAux(s, normalizer, externals, maxSteps, useExternals));
        const newTerm = reducer.oneStepReduce(reduced);
        maxSteps.steps--;
        if (newTerm === undefined)
            return reduced;
        reduced = newTerm;
    }
    return reduced;
};
const traceNormalize = (query, normalizer, externals, trace) => {
    let reduced = query;
    if (Utils_1.isVar(query)) {
        trace(query);
        return query;
    }
    const reducer = exports.oneStepReduceWithExternals(normalizer, externals);
    while (true) {
        if (Utils_1.isVar(reduced))
            return reduced;
        Utils_2.mapMut(reduced.args, (s, i) => exports.traceNormalize(s, normalizer, externals, (t) => trace(Utils_1.replaceTermAt(reduced, t, [i]))));
        const newTerm = reducer.oneStepReduce(reduced);
        if (newTerm === undefined)
            return reduced;
        reduced = newTerm;
        trace(reduced);
    }
};
exports.traceNormalize = traceNormalize;
const buildNormalizer = (evaluator, externals = {}, useExternals = true) => (query) => {
    return exports.normalize(query, evaluator, externals, useExternals);
};
exports.buildNormalizer = buildNormalizer;
const buildFueledNormalizer = (evaluator, maxSteps, externals = {}, useExternals = true) => (query) => {
    return exports.normalizeWithFuel(query, evaluator, externals, maxSteps, useExternals);
};
exports.buildFueledNormalizer = buildFueledNormalizer;
const makeNormalizerAsync = (normalizer) => {
    return (query) => Promise.resolve(normalizer(query));
};
exports.makeNormalizerAsync = makeNormalizerAsync;
const normalizeQuery = (query, source, externals, normalizerFactory, passes = Utils_1.defaultPasses(externals('native'))) => __awaiter(void 0, void 0, void 0, function* () {
    const querySymb = "___query";
    const queryVars = Utils_1.uniq(Utils_1.vars(query));
    const queryLhs = Utils_1.fun(querySymb, ...queryVars);
    const queryRule = Utils_1.showRule([queryLhs, query]);
    const sourceWithQuery = `${source}\n${queryRule}`;
    const trs = yield Unification_1.compileRules(sourceWithQuery, passes);
    if (Utils_1.isNothing(trs))
        return;
    const normalize = normalizerFactory(trs, externals);
    const start = Date.now();
    const nf = yield normalize(queryLhs);
    const delta = Date.now() - start;
    return { duration: delta, normalForm: nf };
});
exports.normalizeQuery = normalizeQuery;
