"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeQueryWith = exports.normalizersList = void 0;
const DecisionTreeNormalizer_1 = require("./DecisionTreeNormalizer");
const Normalizer_1 = require("./Normalizer");
const WebWorkerNormalizer_1 = require("./JSNormalizer/WebWorkerNormalizer");
const Unification_1 = require("./Unification");
const HeadMatcher_1 = require("./Matchers/HeadMatcher");
const ClosureMatcher_1 = require("./Matchers/ClosureMatcher/ClosureMatcher");
const Utils_1 = require("../Compiler/Utils");
exports.normalizersList = [
    'decision-trees',
    'web-worker',
    'head-matcher',
    'closure-matcher'
];
const normalizeQueryWith = (normalizer) => {
    const factory = {
        'decision-trees': (trs, externals) => Normalizer_1.makeNormalizerAsync(new DecisionTreeNormalizer_1.DecisionTreeNormalizer(trs).asNormalizer(externals('native'))),
        'head-matcher': (trs, externals) => Normalizer_1.makeNormalizerAsync(Normalizer_1.buildNormalizer(Unification_1.unificationNormalizer(HeadMatcher_1.headMatcher(trs)), externals('native'))),
        'closure-matcher': (trs, externals) => Normalizer_1.makeNormalizerAsync(new ClosureMatcher_1.ClosureMatcher(trs).asNormalizer(externals('native'))),
        'web-worker': (trs, externals) => WebWorkerNormalizer_1.webWorkerNormalizer(trs, externals('js'))
    }[normalizer];
    return (query, source, externals, passes = Utils_1.defaultPasses(externals('native'))) => Normalizer_1.normalizeQuery(query, source, externals, factory, passes);
};
exports.normalizeQueryWith = normalizeQueryWith;
