"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ruleBasedUnify = void 0;
const Utils_1 = require("../Compiler/Utils");
const Types_1 = require("../Parser/Types");
const Utils_2 = require("../Parser/Utils");
const ruleBasedUnify = (s, t) => unify([[s, t]]);
exports.ruleBasedUnify = ruleBasedUnify;
// http://moscova.inria.fr/~levy/courses/X/IF/03/pi/levy2/martelli-montanari.pdf
// We use directed unification so we don't need the orient/swap rule
const unify = (eqs) => {
    const sigma = {};
    while (eqs.length > 0) {
        const [s, t] = eqs.pop();
        // Decompose
        if (Utils_1.isFun(s) && Utils_1.isFun(t, s.name) &&
            s.args.length === t.args.length) {
            eqs.push(...Utils_1.zip(s.args, t.args));
            continue;
        }
        // Delete (at this point s === t iff isVar(s) && isVar(t, s))
        if (s === t)
            continue;
        // Eliminate
        if (Utils_1.isVar(s) && !Utils_1.occurs(s, t)) {
            Utils_2.mapMut(eqs, eq => Utils_2.mapMut(eq, u => Utils_1.substitute(u, sigma)));
            Types_1.dictSet(sigma, s, t);
            continue;
        }
        return; // not unifyable
    }
    return sigma;
};
