"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.unificationNormalizer = exports.oneStepReducer = exports.compileRules = exports.matches = exports.match = void 0;
const CompilerPass_1 = require("../Compiler/Passes/CompilerPass");
const Utils_1 = require("../Compiler/Utils");
const Parser_1 = require("../Parser/Parser");
const RemoveComments_1 = require("../Parser/Preprocessor/RemoveComments");
const Types_1 = require("../Types");
const RuleBasedUnify_1 = require("./RuleBasedUnify");
const match = (s, t) => RuleBasedUnify_1.ruleBasedUnify(t, s);
exports.match = match;
const matches = (s, t) => Utils_1.isSomething(exports.match(s, t));
exports.matches = matches;
function logErrors(errors) {
    for (const err of Types_1.unwrap(errors)) {
        console.error(err);
    }
}
function compileRules(src, passes) {
    return __awaiter(this, void 0, void 0, function* () {
        const rules = yield Parser_1.parse(src, RemoveComments_1.removeComments);
        if (Types_1.isError(rules)) {
            logErrors(rules);
            return;
        }
        const trs = CompilerPass_1.compile(Types_1.unwrap(rules).trs, ...passes);
        if (Types_1.isError(trs)) {
            logErrors(trs);
            return;
        }
        return Types_1.unwrap(trs);
    });
}
exports.compileRules = compileRules;
const oneStepReducer = (matcher) => (term) => {
    const matched = matcher(term, exports.match);
    if (matched) {
        return Utils_1.substitute(Utils_1.rhs(matched.rule), matched.sigma);
    }
};
exports.oneStepReducer = oneStepReducer;
const unificationNormalizer = (matcher) => ({
    oneStepReduce: exports.oneStepReducer(matcher)
});
exports.unificationNormalizer = unificationNormalizer;
