"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.format = exports.indent = exports.tabs = exports.repeatString = exports.gen = exports.pop = exports.traverseNames = exports.traverse = exports.time = exports.mapString = exports.mapMut = exports.setFilter = exports.setMap = exports.intersectionMut = exports.unionMut = exports.intersection = exports.union = exports.repeat = exports.map = exports.range = exports.indexed = exports.partition = exports.find = exports.every = exports.some = exports.iter = exports.once = exports.join = exports.reverse = void 0;
const Utils_1 = require("../Compiler/Utils");
function* reverse(elems) {
    for (let i = elems.length - 1; i >= 0; i--) {
        yield elems[i];
    }
}
exports.reverse = reverse;
function* join(as, bs) {
    for (const a of as)
        yield a;
    for (const b of bs)
        yield b;
}
exports.join = join;
function* once(val) {
    yield val;
}
exports.once = once;
function* iter(vals) {
    for (const val of vals) {
        yield val;
    }
}
exports.iter = iter;
function some(it, pred) {
    for (const val of it) {
        if (pred(val))
            return true;
    }
    return false;
}
exports.some = some;
function every(it, pred) {
    for (const val of it) {
        if (!pred(val))
            return false;
    }
    return true;
}
exports.every = every;
function find(it, pred) {
    for (const val of it) {
        if (pred(val))
            return val;
    }
}
exports.find = find;
function partition(it, pred) {
    const as = [];
    const bs = [];
    for (const val of it) {
        if (pred(val)) {
            as.push(val);
        }
        else {
            bs.push(val);
        }
    }
    return [as, bs];
}
exports.partition = partition;
function* indexed(vals) {
    let i = 0;
    for (const val of vals) {
        yield [val, i++];
    }
}
exports.indexed = indexed;
function* range(from, to, step = 1) {
    for (let i = from; i <= to; i += step) {
        yield i;
    }
}
exports.range = range;
function* map(it, f) {
    for (const val of it) {
        yield f(val);
    }
}
exports.map = map;
function* repeat(val, count) {
    for (let i = 0; i < count; i++) {
        yield val;
    }
}
exports.repeat = repeat;
function union(...sets) {
    return unionMut(new Set(), ...sets);
}
exports.union = union;
function intersection(...sets) {
    return intersectionMut(new Set(), ...sets);
}
exports.intersection = intersection;
function unionMut(a, ...sets) {
    for (const set of sets) {
        for (const val of set) {
            a.add(val);
        }
    }
    return a;
}
exports.unionMut = unionMut;
function intersectionMut(a, ...sets) {
    for (const set of sets) {
        for (const val of a) {
            if (!set.has(val)) {
                a.delete(val);
            }
        }
    }
    return a;
}
exports.intersectionMut = intersectionMut;
function setMap(set, f) {
    return new Set([...set.values()].map(f));
}
exports.setMap = setMap;
function setFilter(set, f) {
    return new Set([...set.values()].filter(f));
}
exports.setFilter = setFilter;
function mapMut(vals, f) {
    for (let i = 0; i < vals.length; i++) {
        vals[i] = f(vals[i], i);
    }
    return vals;
}
exports.mapMut = mapMut;
function mapString(str, f) {
    let newStr = '';
    for (const char of str) {
        newStr += f(char);
    }
    return newStr;
}
exports.mapString = mapString;
function time(f) {
    const start = Date.now();
    const res = f();
    return [Date.now() - start, res];
}
exports.time = time;
function* traverse(term) {
    if (Utils_1.isVar(term)) {
        yield term;
    }
    else {
        yield term;
        for (const arg of term.args) {
            for (const t of traverse(arg)) {
                yield t;
            }
        }
    }
}
exports.traverse = traverse;
function* traverseNames(term) {
    for (const t of traverse(term)) {
        if (Utils_1.isVar(t)) {
            yield t;
        }
        else {
            yield t.name;
        }
    }
}
exports.traverseNames = traverseNames;
function pop(elems, count = 1) {
    const popped = [];
    for (let i = 0; i < count; i++) {
        popped.push(elems.pop());
    }
    return popped;
}
exports.pop = pop;
function* gen(count, f) {
    for (let i = 0; i < count; i++) {
        yield f(i);
    }
}
exports.gen = gen;
const repeatString = (str, n) => {
    if (n < 1)
        return '';
    let seq = str;
    while (2 * seq.length <= n) {
        seq += seq;
    }
    seq += seq.slice(0, n - seq.length);
    return seq;
};
exports.repeatString = repeatString;
const tabs = (count) => exports.repeatString('   ', count);
exports.tabs = tabs;
const indent = (tabCount, ...lines) => {
    const ident = exports.tabs(tabCount);
    return lines.map(l => `${ident}${l}`).join('\n');
};
exports.indent = indent;
const format = (...lines) => {
    let tabCount = 0;
    const indented = [];
    for (const line of map(lines, s => s.trimRight())) {
        if (line.endsWith('{')) {
            indented.push(exports.indent(tabCount, line));
            tabCount++;
            continue;
        }
        else if (line.endsWith('}')) {
            tabCount--;
        }
        indented.push(exports.indent(tabCount, line));
    }
    return indented.join('\n');
};
exports.format = format;
