"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HaskellTranslator = void 0;
const Utils_1 = require("../Compiler/Utils");
const Translator_1 = require("./Translator");
const nat = (n) => `(Nat ${n})`;
class HaskellTranslator extends Translator_1.Translator {
    init() {
        this.setReservedKeywords(HaskellTranslator.reservedKeywords);
        this.header = [
            "import Data.List (intercalate, map)",
            "data Term = Var String | Fun String [Term] | Nat Int deriving (Eq)"
        ];
    }
    declareNullary(varName, symb) {
        return `${varName} = Fun "${symb}" []`;
    }
    translateTerm(term, useNullaryVars = false) {
        if (Utils_1.isVar(term))
            return this.renameVar(term);
        if (Translator_1.isNat(term.name))
            return nat(term.name);
        if (useNullaryVars && term.args.length === 0 && this.nullaries.has(term.name)) {
            return Translator_1.nullaryVarName(term.name);
        }
        return `(Fun "${term.name}" [${term.args.map(t => this.translateTerm(t)).join(", ")}])`;
    }
    callTerm(term) {
        if (Utils_1.isVar(term))
            return term;
        if (!this.isDefined(term.name)) {
            return this.translateTerm(Utils_1.fun(term.name, ...term.args.map(t => this.callTerm(t))), true);
        }
        const args = `${term.args.map(t => this.callTerm(t)).join(' ')}`;
        return `(${term.name} ${args})`;
    }
    translateRules(name, rules) {
        const res = [];
        for (const [lhs, rhs] of rules) {
            const args = `${lhs.args.map(t => this.translateTerm(t)).join(' ')}`;
            res.push(`${name} ${args} = ${this.callTerm(rhs)}`);
        }
        if (!Utils_1.hasMostGeneralRule(rules)) {
            const args = Utils_1.genVars(Utils_1.arity(rules));
            res.push(`${name} ${args.join(' ')} = ${this.translateTerm(Utils_1.fun(name, ...args), true)}`);
        }
        return res.join('\n');
    }
}
exports.HaskellTranslator = HaskellTranslator;
HaskellTranslator.reservedKeywords = new Set([
    'case', 'class', 'data', 'deriving', 'do', 'else', 'if',
    'import', 'in', 'infix', 'infixl', 'infixr', 'instance',
    'let', 'of', 'module', 'newtype', 'then', 'type', 'where'
]);
