"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JSTranslator = void 0;
const Utils_1 = require("../Parser/Utils");
const ImperativeLangTranslator_1 = require("./ImperativeLangTranslator");
const Translator_1 = require("./Translator");
class JSTranslator extends ImperativeLangTranslator_1.ImperativeLangTranslator {
    init() {
        this.header.push(Utils_1.format('function isFun(term) {', 'return typeof term === "object";', '}'), Utils_1.format('function isVar(term) {', 'return typeof term === "string";', '}'), Utils_1.format('function isNat(term) {', 'const t = typeof term;', 'return t === "number" || t === "bigint";', '}'));
    }
    translateAst(ast) {
        const tr = this.translateAst.bind(this);
        switch (ast.type) {
            case 'var':
                return ast.name;
            case 'assign':
                return `${ast.lhs} = ${ast.rhs};`;
            case 'block':
                return `{
                    ${ast.statements.map(s => tr(s)).join('\n')}
                }`;
            case 'const_decl':
                return `const ${ast.name} = ${ast.value};`;
            case 'continue':
                return 'continue;';
            case 'fun':
                if (Translator_1.isNat(ast.name))
                    return ast.name;
                return `{ name: "${ast.name}", args: [${ast.args.map(a => tr(a)).join(', ')}] }`;
            case 'function_call':
                return `${ast.funName}(${ast.args.map(a => tr(a)).join(', ')})`;
            case 'function_decl':
                return `function ${ast.name}(${ast.argNames.join(', ')}) {
                    ${tr(ast.body)}
                }`;
            case 'infinite_loop':
                return `while (true) {
                    ${tr(ast.body)}
                }`;
            case 'nat':
                return `${ast.value}`;
            case 'raw':
                return ast.value;
            case 'return':
                return `return ${tr(ast.value)};`;
            case 'string':
                return `"${ast.value}"`;
            case 'subterm_access':
                return `${tr(ast.parent)}.args[${ast.childIndex}]`;
            case 'switch':
                return `switch (${tr(ast.testedValue)}) {
                    ${ast.cases.map(c => {
                    if (c.type === 'case') {
                        return `case ${tr(c.test)}:
                                ${tr(c.body)}
                            `;
                    }
                    else {
                        return `default: 
                            ${tr(c.body)}
                        `;
                    }
                }).join('\n')}
                }`;
            case 'term_name':
                const t = tr(ast.term);
                return `${t}.name || ${t}`;
        }
    }
}
exports.JSTranslator = JSTranslator;
