import { Dict, Rule, Symb, Term } from "../../Parser/Types";
import { DecisionTree } from "./DecisionTree";
export declare type AnyPat = '_';
export declare const _: AnyPat;
export declare type FunPattern = {
    name: Symb;
    args: Pattern[];
};
export declare type Pattern = FunPattern | AnyPat;
declare type ClauseMatrixRow = Pattern[];
export declare type ClauseMatrix = {
    dims: [number, number];
    patterns: ClauseMatrixRow[];
    actions: OccTerm[];
};
export declare const patternOf: (term: Term) => Pattern;
export declare const occurencesOf: (term: Term, sigma?: Dict<IndexedOccurence>, subTermIndex?: number) => Dict<IndexedOccurence>;
export declare type OccTerm = IndexedOccurence | {
    name: Symb;
    args: OccTerm[];
};
export declare const substituteOccurences: (term: Term, occs: Dict<IndexedOccurence>) => OccTerm;
export declare const occTermOfRule: ([lhs, rhs]: Rule) => OccTerm;
export declare const clauseMatrixOf: (rules: Rule[]) => ClauseMatrix;
export declare const specializeClauseMatrix: (matrix: ClauseMatrix, c: Symb, arity: number) => ClauseMatrix;
export declare const defaultClauseMatrix: (matrix: ClauseMatrix) => ClauseMatrix;
declare type ColumnSelectionHeuristic = (matrix: ClauseMatrix) => number[];
export declare const selectColumn: (matrix: ClauseMatrix) => number;
export declare const heuristicNeededPrefix: ColumnSelectionHeuristic;
export declare const heuristicSmallBranchingFactor: ColumnSelectionHeuristic;
export declare const heuristicArity: ColumnSelectionHeuristic;
export declare const heuristicConstructorPrefix: ColumnSelectionHeuristic;
export declare const isPatternMoreGeneral: (p1: Pattern, p2: Pattern) => boolean;
export declare const isPatternRedundant: (patternIndex: number, patterns: Pattern[]) => boolean;
export declare const isRowUseless: (matrix: ClauseMatrix, rowIndex: number) => boolean;
export declare const isNeeded: (row: number, col: number, matrix: ClauseMatrix) => boolean;
export declare type Occcurence = {
    term: Term;
    pos: number[];
};
export declare type IndexedOccurence = {
    index: number;
    pos: number[];
};
export declare const compileClauseMatrix: (argsCount: number, matrix: ClauseMatrix, signature: Set<Symb>, heuristics?: ColumnSelectionHeuristic[]) => DecisionTree;
export {};
