import { Maybe } from "../Compiler/Utils";
import { ExternalsFactory, NativeExternals } from "../Externals/Externals";
import { Term, TRS } from "../Parser/Types";
export interface StepNormalizer {
    /**
     * Performs one step of reduction when a rule applies
     */
    oneStepReduce: (query: Term) => Maybe<Term>;
}
export interface OneShotNormalizer {
    /**
     * Normalizes query
     */
    normalize: (query: Term) => Promise<Term>;
}
export declare type Normalizer = (query: Term) => Term;
export declare type AsyncNormalizer = (query: Term) => Promise<Term>;
export declare const oneStepReduceWithExternals: (normalizer: StepNormalizer, externals?: NativeExternals<string>) => StepNormalizer;
export declare const normalize: <Externals extends string = string>(query: Term, normalizer: StepNormalizer, externals: NativeExternals<Externals>, useExternals?: boolean) => Term;
export declare const normalizeWithFuel: <Externals extends string = string>(query: Term, normalizer: StepNormalizer, externals: NativeExternals<Externals>, maxSteps: number, useExternals?: boolean) => Term;
export declare const traceNormalize: <Externals extends string = string>(query: Term, normalizer: StepNormalizer, externals: NativeExternals<Externals>, trace: (t: Term) => void) => Term;
export declare const buildNormalizer: (evaluator: StepNormalizer, externals?: NativeExternals<string>, useExternals?: boolean) => Normalizer;
export declare const buildFueledNormalizer: (evaluator: StepNormalizer, maxSteps: number, externals?: NativeExternals<string>, useExternals?: boolean) => Normalizer;
export declare type NormalizerFactory<Exts extends string> = (trs: TRS, externals: ExternalsFactory<Exts>) => AsyncNormalizer;
export declare const makeNormalizerAsync: (normalizer: Normalizer) => AsyncNormalizer;
export declare const normalizeQuery: <Exts extends string>(query: Term, source: string, externals: ExternalsFactory<Exts, "native" | import("../Externals/Externals").Targets>, normalizerFactory: NormalizerFactory<Exts>, passes?: import("../lib").CompilerPass[]) => Promise<Maybe<{
    duration: number;
    normalForm: Term;
}>>;
