import { DecisionTree } from "../Compiler/DecisionTrees/DecisionTree";
import { DecisionTreeTranslator } from "../Compiler/DecisionTrees/DecisionTreeTranslator";
import { Targets } from "../Externals/Externals";
import { Symb, Term, Var } from "../Parser/Types";
import { SourceCode } from './Translator';
export declare type Ast = ConstVarDecl | VariableUse | NatExpr | FunTermUse | FunctionCall | FunctionDecl | Block | SwitchStatement | ReturnStatement | AssignmentStatement | ContinueStatement | SubtermAccess | StringExpr | SubtermAccess | RawSourceCode | TermName | InfiniteLoop;
declare type RawSourceCode = {
    type: 'raw';
    value: SourceCode;
};
declare type SwitchCase = DefaultSwitchCase | NormalSwitchCase;
declare type SubtermAccess = {
    type: 'subterm_access';
    parent: Ast;
    childIndex: number;
};
declare type DefaultSwitchCase = {
    type: 'default';
    body: Ast;
};
declare type NormalSwitchCase = {
    type: 'case';
    test: Ast;
    body: Ast;
};
export declare type SwitchStatement = {
    type: 'switch';
    testedValue: Ast;
    cases: SwitchCase[];
};
export declare type InfiniteLoop = {
    type: 'infinite_loop';
    body: Ast;
};
export declare type FunTermUse = {
    type: 'fun';
    name: Symb;
    args: Ast[];
};
export declare type NatExpr = {
    type: 'nat';
    value: number;
};
export declare type TermName = {
    type: 'term_name';
    term: Ast;
};
export declare type StringExpr = {
    type: 'string';
    value: string;
};
export declare type VariableUse = {
    type: 'var';
    name: string;
};
export declare type ReturnStatement = {
    type: 'return';
    value: Ast;
};
export declare type ContinueStatement = {
    type: 'continue';
};
export declare type ConstVarDecl = {
    type: 'const_decl';
    name: string;
    value: SourceCode;
};
export declare type FunctionDecl = {
    type: 'function_decl';
    name: string;
    argNames: Var[];
    body: Ast;
};
export declare type FunctionCall = {
    type: 'function_call';
    funName: string;
    args: Ast[];
};
export declare type AssignmentStatement = {
    type: 'assign';
    lhs: SourceCode;
    rhs: SourceCode;
};
export declare type Block = {
    type: 'block';
    statements: Ast[];
};
declare type FunCall = {
    funName: string;
    args: ExtendedTerm[];
};
declare type ExtendedTerm = FunCall | Term;
export declare abstract class ImperativeLangTranslator<T extends Targets, Exts extends string> extends DecisionTreeTranslator<T, Exts> {
    protected abstract translateAst(ast: Ast): SourceCode;
    protected declareNullary(varName: string, symb: string): SourceCode;
    translateTerm(term: Term, useNullaryVars?: boolean): SourceCode;
    accessSubterm(parent: SourceCode, childIndex: number): SourceCode;
    translateExtendedTerm(t: ExtendedTerm): SourceCode;
    private translateOccurence;
    private callOccurence;
    private astOfTerm;
    private astOfExtendedTerm;
    private astOfOccurence;
    private compileDecisionTree;
    translateDecisionTree(name: string, dt: DecisionTree, varNames: Var[]): SourceCode;
    callTerm(term: Term): SourceCode;
}
export {};
