import { Maybe } from "../Compiler/Utils";
import { SpecialCharacters } from "../Parser/Lexer/SpecialChars";
import { Rule, Symb, Term, TRS, Var } from "../Parser/Types";
import { Targets, Externals } from "../Externals/Externals";
export declare type SourceCode = string;
export declare const symbolMap: {
    [key in SpecialCharacters]: string;
};
export declare const isNat: (str: string) => boolean;
export declare const nullarySymbolsPrefix = "symb_";
export declare const nullaryVarName: (f: string) => string;
export declare abstract class Translator<Target extends Targets, Exts extends string> {
    protected header: string[];
    protected externals: Externals<Target, Exts>;
    protected trs: TRS;
    protected definedSymbols: Set<Symb>;
    protected reservedKeywords: Set<string>;
    protected nullaries: Set<string>;
    constructor(trs: TRS, externals: Externals<Target, Exts>, reservedKeywords?: Set<string>);
    protected init(): void;
    protected declareNullarySymbols(): void;
    protected abstract declareNullary(varName: string, symb: string): SourceCode;
    protected setReservedKeywords(kw: Set<string>): void;
    withSpecialChars(str: string): string;
    private withPrefix;
    private withoutPrefix;
    private withoutSpecialChars;
    parseRenamedTerm(term: string): Maybe<Term>;
    abstract translateTerm(term: Term, useNullaryVars?: boolean): SourceCode;
    abstract translateRules(name: string, rules: Rule[]): SourceCode;
    isDefined(f: Symb): boolean;
    protected renameVar(v: Var): Var;
    renameTerm(term: Term): Term;
    private renameFun;
    private renameRule;
    private generateExternals;
    private generateRules;
    translate(): SourceCode;
}
